#include <locale.h>
#include <config.h>
#include <gst/player/i18n-support.h>
#include <gnome.h>

#include <gst/player/gstmediaplay.h>

#define GST_PLAYER_DISCLAIMER \
  "Warning ! GStreamer Media Player is highly alpha code !\n \
   This means :\n \
   - you don't need to tell us it sucks\n \
   - you shouldn't remove xine/mplayer/Windows Media Player yet\n \
   - you are welcome to do patches or suggest features\n \
   - you should make liberal use of http://bugzilla.gnome.org/ for bugs\n \
   - you should attach output of gst-feedback to your bug report\n \
   - you shouldn't expect it to play two media files in a row\n \
   \n \
   In general, this is only a test bed for GStreamer features, and not yet \
   the finished product we intend to ship.\n \
   Check the README for more information.\n"



typedef struct {
	GtkWidget *selection;
	GstMediaPlay *mplay;
} file_select;

static void
on_load_file_selected (GtkWidget *button, file_select *data)
{
	GtkWidget *file_selection;
	GstMediaPlay *mplay;

	file_selection = data->selection;
	mplay = data->mplay;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	mplay->uri = NULL;
	gst_media_play_clear_uri_list (mplay);
	gst_media_play_add_uri (mplay, gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selection)));
}

void
cb_open (GtkWidget *widget, GstMediaPlay *mplay)
{
	GtkWidget *file_selector;

	file_select *file_data = g_new0 (file_select, 1);

	g_return_if_fail (GST_IS_MEDIA_PLAY(mplay));

	file_selector = gtk_file_selection_new (_("Please select a file to load."));
	
	file_data->selection = file_selector;
	file_data->mplay = mplay;

	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
			  "clicked", G_CALLBACK (on_load_file_selected), file_data);
  
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
				   "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
				   (gpointer) file_selector);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
				   "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
				   (gpointer) file_selector);
	
	gtk_widget_show (file_selector);
}

void
cb_destroy (GObject object, GtkWidget *window)
{
	gtk_main_quit ();
}

static void
toggle_play_pause (GtkWidget *widget, GstMediaPlay *mplay)
{
	GstElementState playstate;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	playstate = gst_media_play_get_state (mplay);

	if (playstate == GST_STATE_PLAYING) {
		gst_media_play_set_state (mplay, GST_STATE_PAUSED);
	}
	else {
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
	}
}

static void
toggle_mini_display_mode (GtkWidget *widget, GstMediaPlay *mplay)
{
	GstMediaPlayMode display_mode;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	display_mode = gst_media_play_get_display_mode (mplay);
	if (display_mode == GST_MEDIA_PLAY_MINI){
		gst_media_play_set_display_mode (mplay, GST_MEDIA_PLAY_NORMAL);
	}
	else {
		gst_media_play_set_display_mode (mplay, GST_MEDIA_PLAY_MINI);
	}
}

static void
toggle_fullscreen (GtkWidget *widget, GstMediaPlay *mplay)
{
	GstMediaPlayMode display_mode;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	display_mode = gst_media_play_get_display_mode (mplay);
	if (display_mode == GST_MEDIA_PLAY_FULLSCREEN){
		gst_media_play_set_display_mode (mplay, GST_MEDIA_PLAY_NORMAL);
	}
	else {
		gst_media_play_set_display_mode (mplay, GST_MEDIA_PLAY_FULLSCREEN);
	}
}

static void
display_mode_change (GstMediaPlay *mplay, GstMediaPlayMode display_mode, GtkWidget *window)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	if (display_mode == GST_MEDIA_PLAY_MINI){
		gtk_widget_hide (GNOME_APP (window)->menubar);
	}
	else {
		gtk_widget_show (GNOME_APP (window)->menubar);
	}
}

static void
half_size (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_video_scale(mplay, 0.5F);
}

static void
full_size (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_video_scale(mplay, 1.0F);
}

static void
double_size (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_video_scale(mplay, 2.0F);
}

static void
show_media_info (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_show_media_info(mplay);
}

static void
show_preferences (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_show_preferences(mplay);
}

static void
show_about (GtkWidget *widget, GstMediaPlay *mplay)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_show_about(mplay);
}

static int
window_key_press_event (GtkWidget *widget, GdkEventKey *event, GstMediaPlay *mplay)
{
	guint state;
	gboolean handled = FALSE;
  
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));

	state = event->state;
  
	switch (event->keyval) {
	case GDK_KP_Add:
		gst_media_play_set_volume(mplay, gst_media_play_get_volume(mplay) + 0.1);
		handled = TRUE;
		break;
	case GDK_KP_Subtract:
		gst_media_play_set_volume(mplay, gst_media_play_get_volume(mplay) - 0.1);
		handled = TRUE;
		break;
	case GDK_space:
		toggle_play_pause (widget, mplay);
		handled = TRUE;
		break;
	case GDK_m:
		toggle_fullscreen (widget, mplay);
		handled = TRUE;
		break;
	case GDK_Return:
		if (state & GDK_MOD1_MASK)
			toggle_fullscreen (widget, mplay);
		handled = TRUE;
		break;
	}
  
	return handled;
}

static GnomeUIInfo file_menu [] = 
{
	GNOMEUIINFO_MENU_OPEN_ITEM (cb_open, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM (cb_destroy, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo view_menu [] =
{
	{
		GNOME_APP_UI_ITEM, N_("Mini Mode"),
		NULL,
		toggle_mini_display_mode, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_T, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Half Size"),
		NULL,
		half_size, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_1, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Original Size"),
		NULL,
		full_size, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_2, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Double Size"),
		NULL,
		double_size, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_3, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Full Screen"),
		NULL,
		toggle_fullscreen, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_4, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] =
{
	GNOMEUIINFO_MENU_ABOUT_ITEM (show_about, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo main_menu [] = 
{
	GNOMEUIINFO_MENU_FILE_TREE (file_menu),
	GNOMEUIINFO_MENU_VIEW_TREE (view_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

int
main (int argc, char **argv)
{
	GstMediaPlay *mplay;
	GtkWidget *window;
	GtkWidget *dialog;	/* disclaimer dialog */

	gst_init (&argc, &argv);
	gnome_init (PACKAGE, VERSION, argc, argv);

	/* i18n initialization */
	//setlocale (LC_ALL, "");
	bindtextdomain (PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (PACKAGE, "UTF-8");
	textdomain (PACKAGE);

	/* gnome window */
	window = gnome_app_new ("gst-player", "GStreamer Player");
	g_return_if_fail(window != NULL);

	gtk_window_set_default_size (GTK_WINDOW (window), 300, -1);
	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);

	dialog = gtk_message_dialog_new (GTK_WINDOW (window), 
	             GTK_DIALOG_DESTROY_WITH_PARENT,
		     GTK_MESSAGE_WARNING,
		     GTK_BUTTONS_CLOSE,
		     GST_PLAYER_DISCLAIMER);

	/* Destroy the dialog when the user responds to it 
	 * (e.g. clicks a button) */
	gtk_widget_show (dialog);
	g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
	                          G_CALLBACK (gtk_widget_destroy),
	                          GTK_OBJECT (dialog));

	mplay = gst_media_play_new ();
	gnome_app_create_menus_with_data (GNOME_APP (window), main_menu, mplay);
	gnome_app_set_contents (GNOME_APP (window), GTK_WIDGET(mplay));
	
	g_signal_connect (G_OBJECT (window), "destroy",
		  G_CALLBACK (cb_destroy), NULL);
	g_signal_connect (G_OBJECT (window), "key-press-event",
			  G_CALLBACK (window_key_press_event), mplay);
	g_signal_connect (G_OBJECT (mplay), "display_mode_change",
			  G_CALLBACK (display_mode_change), window);

	gtk_widget_show (GTK_WIDGET(mplay));
	gtk_widget_show (window);

	if (argc > 1) {
		int i;

		for (i=1;i<argc;i++) {
			gst_media_play_add_uri (mplay, argv[i]);
		}
	}

	gtk_main ();

	return 0;
}
 

